import json
import os
import sys
import time

from kombu import Message, Connection

from .mocks import Transport

sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
from writeinblob import write_in_blob  # noqa: E402
from .container import TestContainer  # noqa: E402

INI_PATH = '../app.ini'


def build_container() -> TestContainer:
    ct: TestContainer = TestContainer()
    ct.config.from_ini(os.path.join(os.path.dirname(__file__), INI_PATH))
    ct.wire(modules=[sys.modules[__name__], write_in_blob])
    ct.init_resources()
    return ct


def test_should_create_instance() -> None:
    ct = build_container()
    body: dict = {'test': 'messages'}
    write: write_in_blob.BlobWriterHandler = ct.blob_writer_service()
    write.setup(params=('from_test', ))
    write.handler(body=json.dumps(body),
                  message=Message(body=json.dumps(body),
                                  channel=Connection(transport=Transport).channel()))
    time.sleep(3)
    ct.shutdown_resources()
    time.sleep(3)



import json
from typing import Any, Tuple
from urllib import parse

from azure.storage.blob import BlobServiceClient, BlobClient
from dependency_injector.wiring import Provide, inject
from devopstoolsdaven.reports.report import Report
from devopstoolsdaven.vault.vault import Vault
from kombu import Message
from messagehandler.message_handler import MessageHandler

from .name_convention import build_blob_name


class BlobWriterHandler(MessageHandler):
    __origin: str = ''

    @inject
    def __init__(self, service_url: str, container: str,
                 vault: Vault = Provide['vault_service'],
                 report: Report = Provide['report_service']) -> None:
        self.__service: str = service_url
        self.__container: str = container
        self.__vault: Vault = vault
        self.__report: Report = report
        self.__blob_service: BlobServiceClient = BlobServiceClient.from_connection_string(
            self.__vault.read_secret(parse.urlparse(self.__service).netloc)['conn_string'])

    def setup(self, params: Tuple[Any, ...]) -> None:
        if len(params) < 1 or not isinstance(params[0], str):
            raise BadParamsException(params=params)
        self.__origin = params[0]

    def handler(self, body: Any, message: Message) -> None:
        self.__report.add_event_with_type(event_type='message received',
                                          record={
                                              'from_queue': self.__origin,
                                              'length': len(message.body),
                                              'headers': message.headers
                                          })
        blob_name: str = build_blob_name(origin=self.__origin)
        self.save_body(blob_name, message.body)
        self.save_headers(blob_name, message.headers)
        self.__report.add_event_with_type(event_type='message processed',
                                          record={
                                              'blob_name': blob_name,
                                              'service_url': self.__service,
                                              'container': self.__container
                                          })
        message.ack()

    def save_headers(self, blob_name: str, headers: dict) -> None:
        content: str = json.dumps(headers)
        client: BlobClient = self.__blob_service.get_blob_client(
            container=self.__container, blob='{}.headers'.format(blob_name))
        client.upload_blob(content, overwrite=True)
        client.close()

    def save_body(self, blob_name: str, body: str) -> None:
        client: BlobClient = self.__blob_service.get_blob_client(
            container=self.__container, blob='{}.body'.format(blob_name))
        client.upload_blob(body, overwrite=True)
        client.close()


class BadParamsException(ValueError):

    def __init__(self, params: Tuple[Any, ...]):
        self.__params = params

    def to_s(self) -> str:
        return ''.join(self.__params)



import logging
from typing import Any, Tuple, Union

from devopstoolsdaven.reports.report import Report
from kombu import Message
from messagehandler.message_handler import MessageHandler


class TrivialHandler(MessageHandler):
    __report: Union[Report, None] = None
    __origin: Union[str, None] = None

    def setup(self, params: Tuple[Any, ...]) -> None:
        self.__origin = params[1]

    def handler(self, body: Any, message: Message) -> None:
        logging.debug('received from {}: {}'.format(self.__origin, str(body)))
        if self.__report is not None:
            self.__report.add_event_with_type(event_type='message received',
                                              record={
                                                  'from_queue': self.__origin,
                                                  'message': body
                                              })
        message.ack()



from setuptools import setup

# noinspection PyUnresolvedReferences
setup(
    author="David Navarro Alvarez",
    author_email="me@davengeo.com",
    description="message handler to write blobs in different cloud storage providers",
    url="https://github.com/davengeo/write-in-blob",
    name="write-in-blob",
    version='0.0.3',
    packages=[
        'writeinblob',
    ],
    install_requires=[
        'azure-storage-blob',
        'dependency-injector>=4.0,<5.0',
        'messagehandler-ifn==0.1.0',
        'devops-tools-daven==0.0.14',
        'kombu'
    ],
    package_data={
        'ini': ['app.ini']
    },
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: European Union Public Licence 1.2 (EUPL 1.2)',
        'Programming Language :: Python :: 3.8',
        'Topic :: System :: Systems Administration',
    ]
)
